@@ -18,7 +18,7 @@
  */
 
 /***************************************************************************
- * Copyright (C) 2017-2021 ZmartZone Holding BV
+ * Copyright (C) 2017-2022 ZmartZone Holding BV
  * Copyright (C) 2013-2017 Ping Identity Corporation
  * All rights reserved.
  *
@@ -43,41 +43,33 @@
  * @Author: Hans Zandbelt - hans.zandbelt@zmartzone.eu
  */
 
-#include "apr_general.h"
-#include "apr_strings.h"
-
-#include <httpd.h>
-#include <http_config.h>
-#include <http_log.h>
-
-#include "../mod_auth_openidc.h"
-
-#include "hiredis/hiredis.h"
+#include "redis.h"
 
 // TODO: proper Redis error reporting (server unreachable etc.)
 
 extern module AP_MODULE_DECLARE_DATA auth_openidc_module;
 
-typedef struct oidc_cache_cfg_redis_t {
-	oidc_cache_mutex_t *mutex;
-	char *host_str;
-	apr_port_t port;
-	char *passwd;
-	int database;
-	struct timeval connect_timeout;
-	struct timeval timeout;
-	redisContext *ctx;
-} oidc_cache_cfg_redis_t;
-
 #define REDIS_CONNECT_TIMEOUT_DEFAULT 5
 #define REDIS_TIMEOUT_DEFAULT 5
 
-/* create the cache context */
-static void *oidc_cache_redis_cfg_create(apr_pool_t *pool) {
-	oidc_cache_cfg_redis_t *context = apr_pcalloc(pool,
-			sizeof(oidc_cache_cfg_redis_t));
-	context->mutex = oidc_cache_mutex_create(pool);
+typedef struct oidc_cache_cfg_redis_ctx_t {
+	char *host_str;
+	apr_port_t port;
+	redisContext *rctx;
+} oidc_cache_cfg_redis_ctx_t;
+
+static oidc_cache_cfg_redis_ctx_t* oidc_cache_redis_cfg_ctx_create(apr_pool_t *pool) {
+	oidc_cache_cfg_redis_ctx_t *context = apr_pcalloc(pool, sizeof(oidc_cache_cfg_redis_ctx_t));
 	context->host_str = NULL;
+	context->port = 0;
+	context->rctx = NULL;
+	return context;
+}
+
+/* create the cache context */
+static oidc_cache_cfg_redis_t* oidc_cache_redis_cfg_create(apr_pool_t *pool) {
+	oidc_cache_cfg_redis_t *context = apr_pcalloc(pool, sizeof(oidc_cache_cfg_redis_t));
+	context->mutex = oidc_cache_mutex_create(pool);
 	context->passwd = NULL;
 	context->database = -1;
 	context->connect_timeout.tv_sec = REDIS_CONNECT_TIMEOUT_DEFAULT;
@@ -88,50 +80,18 @@ static void *oidc_cache_redis_cfg_create(apr_pool_t *pool) {
 	return context;
 }
 
-/*
- * initialize the Redis struct the specified Redis server
- */
-static int oidc_cache_redis_post_config(server_rec *s) {
-	oidc_cfg *cfg = (oidc_cfg *) ap_get_module_config(s->module_config,
-			&auth_openidc_module);
-
-	if (cfg->cache_cfg != NULL)
-		return APR_SUCCESS;
-	oidc_cache_cfg_redis_t *context = oidc_cache_redis_cfg_create(
-			s->process->pool);
+int oidc_cache_redis_post_config(server_rec *s, oidc_cfg *cfg, const char *name) {
+	oidc_cache_cfg_redis_t *context = oidc_cache_redis_cfg_create(s->process->pool);
 	cfg->cache_cfg = context;
 
-	apr_status_t rv = APR_SUCCESS;
-
 	/* parse the host:post tuple from the configuration */
 	if (cfg->cache_redis_server == NULL) {
-		oidc_serror(s,
-				"cache type is set to \"redis\", but no valid " OIDCRedisCacheServer " setting was found");
+		oidc_serror(s, "cache type is set to \"redis\", but no valid " OIDCRedisCacheServer " setting was found");
 		return HTTP_INTERNAL_SERVER_ERROR;
 	}
 
-	char* scope_id;
-	rv = apr_parse_addr_port(&context->host_str, &scope_id, &context->port,
-			cfg->cache_redis_server, s->process->pool);
-	if (rv != APR_SUCCESS) {
-		oidc_serror(s, "failed to parse cache server: '%s'",
-				cfg->cache_redis_server);
-		return HTTP_INTERNAL_SERVER_ERROR;
-	}
-
-	if (context->host_str == NULL) {
-		oidc_serror(s,
-				"failed to parse cache server, no hostname specified: '%s'",
-				cfg->cache_redis_server);
-		return HTTP_INTERNAL_SERVER_ERROR;
-	}
-
-	if (context->port == 0)
-		context->port = 6379;
-
 	if (cfg->cache_redis_password != NULL) {
-		context->passwd = apr_pstrdup(s->process->pool,
-				cfg->cache_redis_password);
+		context->passwd = apr_pstrdup(s->process->pool, cfg->cache_redis_password);
 	}
 
 	if (cfg->cache_redis_database != -1)
@@ -143,19 +103,83 @@ static int oidc_cache_redis_post_config(server_rec *s) {
 	if (cfg->cache_redis_timeout != -1)
 		context->timeout.tv_sec = cfg->cache_redis_timeout;
 
-	if (oidc_cache_mutex_post_config(s, context->mutex, "redis") == FALSE)
+	if (oidc_cache_mutex_post_config(s, context->mutex, name) == FALSE)
 		return HTTP_INTERNAL_SERVER_ERROR;
 
 	return OK;
 }
 
+static apr_status_t oidc_cache_redis_connect(request_rec *r, oidc_cache_cfg_redis_t *context);
+
+/*
+ * free resources allocated for the per-process Redis connection context
+ */
+static apr_status_t oidc_cache_redis_disconnect(oidc_cache_cfg_redis_t *context) {
+	oidc_cache_cfg_redis_ctx_t *rctx = NULL;
+	if (context != NULL) {
+		rctx = (oidc_cache_cfg_redis_ctx_t*) context->ctx;
+		if ((rctx != NULL) && (rctx->rctx != NULL)) {
+			redisFree(rctx->rctx);
+			rctx->rctx = NULL;
+		}
+	}
+	return APR_SUCCESS;
+}
+
+/*
+ * initialize the Redis struct the specified Redis server
+ */
+static int oidc_cache_redis_post_config_impl(server_rec *s) {
+	apr_status_t rv = APR_SUCCESS;
+	oidc_cache_cfg_redis_t *context = NULL;
+	oidc_cache_cfg_redis_ctx_t *rctx = NULL;
+	oidc_cfg *cfg = (oidc_cfg*) ap_get_module_config(s->module_config, &auth_openidc_module);
+
+	if (cfg->cache_cfg != NULL)
+		return OK;
+
+	if (oidc_cache_redis_post_config(s, cfg, "redis") != OK)
+		return HTTP_INTERNAL_SERVER_ERROR;
+
+	context = cfg->cache_cfg;
+	rctx = oidc_cache_redis_cfg_ctx_create(s->process->pool);
+	context->ctx = rctx;
+
+	/* parse the host:post tuple from the configuration */
+	if (cfg->cache_redis_server == NULL) {
+		oidc_serror(s, "cache type is set to \"redis\", but no valid " OIDCRedisCacheServer " setting was found");
+		return HTTP_INTERNAL_SERVER_ERROR;
+	}
+
+	char *scope_id;
+	rv =
+			apr_parse_addr_port(&rctx->host_str, &scope_id, &rctx->port, cfg->cache_redis_server, s->process->pool);
+	if (rv != APR_SUCCESS) {
+		oidc_serror(s, "failed to parse cache server: '%s'", cfg->cache_redis_server);
+		return HTTP_INTERNAL_SERVER_ERROR;
+	}
+
+	if (rctx->host_str == NULL) {
+		oidc_serror(s, "failed to parse cache server, no hostname specified: '%s'", cfg->cache_redis_server);
+		return HTTP_INTERNAL_SERVER_ERROR;
+	}
+
+	if (rctx->port == 0)
+		rctx->port = 6379;
+
+	context->connect = oidc_cache_redis_connect;
+	context->command = oidc_cache_redis_command;
+	context->disconnect = oidc_cache_redis_disconnect;
+
+	return OK;
+}
+
 /*
  * initialize the Redis cache in a child process
  */
 int oidc_cache_redis_child_init(apr_pool_t *p, server_rec *s) {
-	oidc_cfg *cfg = ap_get_module_config(s->module_config,
-			&auth_openidc_module);
-	oidc_cache_cfg_redis_t *context = (oidc_cache_cfg_redis_t *) cfg->cache_cfg;
+	oidc_cfg *cfg = ap_get_module_config(s->module_config, &auth_openidc_module);
+	oidc_cache_cfg_redis_t *context = (oidc_cache_cfg_redis_t*) cfg->cache_cfg;
 
 	/* initialize the lock for the child process */
 	return oidc_cache_mutex_child_init(p, s, context->mutex);
@@ -164,22 +188,10 @@ int oidc_cache_redis_child_init(apr_pool_t *p, server_rec *s) {
 /*
  * assemble single key name based on section/key input
  */
-static char *oidc_cache_redis_get_key(apr_pool_t *pool, const char *section,
-		const char *key) {
+static char* oidc_cache_redis_get_key(apr_pool_t *pool, const char *section, const char *key) {
 	return apr_psprintf(pool, "%s:%s", section, key);
 }
 
-/*
- * free resources allocated for the per-process Redis connection context
- */
-static apr_status_t oidc_cache_redis_free(oidc_cache_cfg_redis_t *context) {
-	if ((context != NULL) && (context->ctx != NULL)) {
-		redisFree(context->ctx);
-		context->ctx = NULL;
-	}
-	return APR_SUCCESS;
-}
-
 /*
  * free and nullify a reply object
  */
@@ -193,70 +205,70 @@ static void oidc_cache_redis_reply_free(redisReply **reply) {
 /*
  * connect to Redis server
  */
-static apr_status_t oidc_cache_redis_connect(request_rec *r,
-		oidc_cache_cfg_redis_t *context) {
+static apr_status_t oidc_cache_redis_connect(request_rec *r, oidc_cache_cfg_redis_t *context) {
 
+	oidc_cache_cfg_redis_ctx_t *rctx = (oidc_cache_cfg_redis_ctx_t*) context->ctx;
 	redisReply *reply = NULL;
 
-	if (context->ctx == NULL) {
+	if (rctx->rctx != NULL)
+		goto end;
 
-		/* no connection, connect to the configured Redis server */
-		oidc_debug(r, "calling redisConnectWithTimeout");
-		context->ctx = redisConnectWithTimeout(context->host_str, context->port, context->connect_timeout);
+	/* no connection, connect to the configured Redis server */
+	oidc_debug(r, "calling redisConnectWithTimeout");
+	rctx->rctx = redisConnectWithTimeout(rctx->host_str, rctx->port, context->connect_timeout);
 
-		/* check for errors */
-		if ((context->ctx == NULL) || (context->ctx->err != 0)) {
-			oidc_error(r, "failed to connect to Redis server (%s:%d): '%s'",
-					context->host_str, context->port,
-					context->ctx != NULL ? context->ctx->errstr : "");
-			oidc_cache_redis_free(context);
-		} else {
-			/* log the connection */
-			oidc_debug(r, "successfully connected to Redis server (%s:%d)",
-					context->host_str, context->port);
-
-			/* see if we need to authenticate to the Redis server */
-			if (context->passwd != NULL) {
-				reply = redisCommand(context->ctx, "AUTH %s", context->passwd);
-				if ((reply == NULL) || (reply->type == REDIS_REPLY_ERROR))
-					oidc_error(r,
-							"Redis AUTH command (%s:%d) failed: '%s' [%s]",
-							context->host_str, context->port,
-							context->ctx->errstr, reply ? reply->str : "<n/a>");
-				else
-					oidc_debug(r,
-							"successfully authenticated to the Redis server: %s",
-							reply ? reply->str : "<n/a>");
-
-				/* free the auth answer */
-				oidc_cache_redis_reply_free(&reply);
-			}
-
-			/* see if we need to set the database */
-			if (context->database != -1) {
-				reply = redisCommand(context->ctx, "SELECT %d",
-						context->database);
-				if ((reply == NULL) || (reply->type == REDIS_REPLY_ERROR))
-					oidc_error(r,
-							"Redis SELECT command (%s:%d) failed: '%s' [%s]",
-							context->host_str, context->port,
-							context->ctx->errstr, reply ? reply->str : "<n/a>");
-				else
-					oidc_debug(r,
-							"successfully selected database %d on the Redis server: %s",
-							context->database, reply ? reply->str : "<n/a>");
-
-				/* free the database answer */
-				oidc_cache_redis_reply_free(&reply);
-			}
-
-			if (redisSetTimeout(context->ctx, context->timeout) != REDIS_OK)
-				oidc_error(r, "redisSetTimeout failed: %s", context->ctx->errstr);
-
-		}
+	/* check for errors */
+	if ((rctx->rctx == NULL) || (rctx->rctx->err != 0)) {
+		oidc_error(r, "failed to connect to Redis server (%s:%d): '%s'", rctx->host_str, rctx->port, rctx->rctx != NULL ? rctx->rctx->errstr : "");
+		context->disconnect(context);
+		goto end;
 	}
 
-	return (context->ctx != NULL) ? APR_SUCCESS : APR_EGENERAL;
+	/* log the connection */
+	oidc_debug(r, "successfully connected to Redis server (%s:%d)", rctx->host_str, rctx->port);
+
+	if (redisSetTimeout(rctx->rctx, context->timeout) != REDIS_OK)
+		oidc_error(r, "redisSetTimeout failed: %s", rctx->rctx->errstr);
+
+	/* see if we need to authenticate to the Redis server */
+	if (context->passwd != NULL) {
+		reply = redisCommand(rctx->rctx, "AUTH %s", context->passwd);
+		if ((reply == NULL) || (reply->type == REDIS_REPLY_ERROR))
+			oidc_error(r, "Redis AUTH command (%s:%d) failed: '%s' [%s]", rctx->host_str, rctx->port, rctx->rctx->errstr,
+					   reply ? reply->str : "<n/a>");
+		else
+			oidc_debug(r, "successfully authenticated to the Redis server: %s",
+					   reply ? reply->str : "<n/a>");
+
+		/* free the auth answer */
+		oidc_cache_redis_reply_free(&reply);
+	}
+
+	/* see if we need to set the database */
+	if (context->database != -1) {
+		reply = redisCommand(rctx->rctx, "SELECT %d", context->database);
+		if ((reply == NULL) || (reply->type == REDIS_REPLY_ERROR))
+			oidc_error(r, "Redis SELECT command (%s:%d) failed: '%s' [%s]", rctx->host_str, rctx->port, rctx->rctx->errstr,
+					   reply ? reply->str : "<n/a>");
+		else
+			oidc_debug(r, "successfully selected database %d on the Redis server: %s", context->database,
+					   reply ? reply->str : "<n/a>");
+
+		/* free the database answer */
+		oidc_cache_redis_reply_free(&reply);
+	}
+
+end:
+
+	return (rctx->rctx != NULL) ? APR_SUCCESS : APR_EGENERAL;
+}
+
+redisReply* oidc_cache_redis_command(request_rec *r, oidc_cache_cfg_redis_t *context, char **errstr,
+		const char *format, va_list ap) {
+	oidc_cache_cfg_redis_ctx_t *rctx = (oidc_cache_cfg_redis_ctx_t*) context->ctx;
+	redisReply *reply = redisvCommand(rctx->rctx, format, ap);
+	*errstr = apr_pstrdup(r->pool, rctx->rctx->errstr);
+	return reply;
 }
 
 #define OIDC_REDIS_MAX_TRIES 2
@@ -264,10 +276,12 @@ static apr_status_t oidc_cache_redis_connect(request_rec *r,
 /*
  * execute Redis command and deal with return value
  */
-static redisReply* oidc_cache_redis_command(request_rec *r,
-		oidc_cache_cfg_redis_t *context, const char *format, ...) {
+static redisReply* oidc_cache_redis_exec(request_rec *r, oidc_cache_cfg_redis_t *context,
+		const char *format, ...) {
 
+	oidc_cache_cfg_redis_ctx_t *rctx = (oidc_cache_cfg_redis_ctx_t*) context->ctx;
 	redisReply *reply = NULL;
+	char *errstr = NULL;
 	int i = 0;
 	va_list ap;
 
@@ -275,12 +289,12 @@ static redisReply* oidc_cache_redis_command(request_rec *r,
 	for (i = 0; i < OIDC_REDIS_MAX_TRIES; i++) {
 
 		/* connect */
-		if (oidc_cache_redis_connect(r, context) != APR_SUCCESS)
+		if (context->connect(r, context) != APR_SUCCESS)
 			break;
 
 		va_start(ap, format);
 		/* execute the actual command */
-		reply = redisvCommand(context->ctx, format, ap);
+		reply = context->command(r, context, &errstr, format, ap);
 		va_end(ap);
 
 		/* check for errors, need to return error replies for cache miss case REDIS_REPLY_NIL */
@@ -289,16 +303,14 @@ static redisReply* oidc_cache_redis_command(request_rec *r,
 			break;
 
 		/* something went wrong, log it */
-		oidc_error(r,
-				"Redis command (attempt=%d to %s:%d) failed, disconnecting: '%s' [%s]",
-				i, context->host_str, context->port, context->ctx->errstr,
+		oidc_error(r, "Redis command (attempt=%d to %s:%d) failed, disconnecting: '%s' [%s]", i, rctx->host_str, rctx->port, errstr,
 				reply ? reply->str : "<n/a>");
 
 		/* free the reply (if there is one allocated) */
 		oidc_cache_redis_reply_free(&reply);
 
 		/* cleanup, we may try again (once) after reconnecting */
-		oidc_cache_redis_free(context);
+		context->disconnect(context);
 	}
 
 	return reply;
@@ -307,12 +319,11 @@ static redisReply* oidc_cache_redis_command(request_rec *r,
 /*
  * get a name/value pair from Redis
  */
-static apr_byte_t oidc_cache_redis_get(request_rec *r, const char *section,
-		const char *key, const char **value) {
+apr_byte_t oidc_cache_redis_get(request_rec *r, const char *section, const char *key,
+		const char **value) {
 
-	oidc_cfg *cfg = ap_get_module_config(r->server->module_config,
-			&auth_openidc_module);
-	oidc_cache_cfg_redis_t *context = (oidc_cache_cfg_redis_t *) cfg->cache_cfg;
+	oidc_cfg *cfg = ap_get_module_config(r->server->module_config, &auth_openidc_module);
+	oidc_cache_cfg_redis_t *context = (oidc_cache_cfg_redis_t*) cfg->cache_cfg;
 	redisReply *reply = NULL;
 	apr_byte_t rv = FALSE;
 
@@ -322,7 +333,7 @@ static apr_byte_t oidc_cache_redis_get(request_rec *r, const char *section,
 
 	/* get */
 	reply =
-			oidc_cache_redis_command(r, context, "GET %s", oidc_cache_redis_get_key(r->pool, section, key));
+			oidc_cache_redis_exec(r, context, "GET %s", oidc_cache_redis_get_key(r->pool, section, key));
 
 	if (reply == NULL)
 		goto end;
@@ -340,11 +351,8 @@ static apr_byte_t oidc_cache_redis_get(request_rec *r, const char *section,
 	}
 
 	/* do a sanity check on the returned value */
-	if ((reply->str == NULL)
-			|| (reply->len != strlen(reply->str))) {
-		oidc_error(r,
-				"redisCommand reply->len (%d) != strlen(reply->str): '%s'",
-				(int )reply->len, reply->str);
+	if ((reply->str == NULL) || (reply->len != strlen(reply->str))) {
+		oidc_error(r, "redisCommand reply->len (%d) != strlen(reply->str): '%s'", (int )reply->len, reply->str);
 		goto end;
 	}
 
@@ -368,12 +376,11 @@ end:
 /*
  * store a name/value pair in Redis
  */
-static apr_byte_t oidc_cache_redis_set(request_rec *r, const char *section,
-		const char *key, const char *value, apr_time_t expiry) {
+apr_byte_t oidc_cache_redis_set(request_rec *r, const char *section, const char *key,
+		const char *value, apr_time_t expiry) {
 
-	oidc_cfg *cfg = ap_get_module_config(r->server->module_config,
-			&auth_openidc_module);
-	oidc_cache_cfg_redis_t *context = (oidc_cache_cfg_redis_t *) cfg->cache_cfg;
+	oidc_cfg *cfg = ap_get_module_config(r->server->module_config, &auth_openidc_module);
+	oidc_cache_cfg_redis_t *context = (oidc_cache_cfg_redis_t*) cfg->cache_cfg;
 	redisReply *reply = NULL;
 	apr_byte_t rv = FALSE;
 	apr_uint32_t timeout;
@@ -387,7 +394,7 @@ static apr_byte_t oidc_cache_redis_set(request_rec *r, const char *section,
 
 		/* delete it */
 		reply =
-				oidc_cache_redis_command(r, context, "DEL %s", oidc_cache_redis_get_key(r->pool, section, key));
+				oidc_cache_redis_exec(r, context, "DEL %s", oidc_cache_redis_get_key(r->pool, section, key));
 
 	} else {
 
@@ -396,7 +403,7 @@ static apr_byte_t oidc_cache_redis_set(request_rec *r, const char *section,
 
 		/* store it */
 		reply =
-				oidc_cache_redis_command(r, context, "SETEX %s %d %s", oidc_cache_redis_get_key(r->pool, section, key), timeout, value);
+				oidc_cache_redis_exec(r, context, "SETEX %s %d %s", oidc_cache_redis_get_key(r->pool, section, key), timeout, value);
 
 	}
 
@@ -412,30 +419,27 @@ static apr_byte_t oidc_cache_redis_set(request_rec *r, const char *section,
 	return rv;
 }
 
-static int oidc_cache_redis_destroy(server_rec *s) {
-	oidc_cfg *cfg = (oidc_cfg *) ap_get_module_config(s->module_config,
-			&auth_openidc_module);
-	oidc_cache_cfg_redis_t *context = (oidc_cache_cfg_redis_t *) cfg->cache_cfg;
+static int oidc_cache_redis_destroy_impl(server_rec *s) {
+	oidc_cfg *cfg = (oidc_cfg*) ap_get_module_config(s->module_config, &auth_openidc_module);
+	oidc_cache_cfg_redis_t *context = (oidc_cache_cfg_redis_t*) cfg->cache_cfg;
 
 	if (context != NULL) {
-		// TODO: why do we need this check...? it is set/checked to null inside of oidc_cache_redis_free, no?
-		if (context->ctx != NULL) {
-			oidc_cache_mutex_lock(s, context->mutex);
-			oidc_cache_redis_free(context);
-			oidc_cache_mutex_unlock(s, context->mutex);
-		}
+		oidc_cache_mutex_lock(s, context->mutex);
+		context->disconnect(context);
+		oidc_cache_mutex_unlock(s, context->mutex);
 		oidc_cache_mutex_destroy(s, context->mutex);
+		cfg->cache_cfg = NULL;
 	}
 
 	return APR_SUCCESS;
 }
 
 oidc_cache_t oidc_cache_redis = {
-		"redis",
-		1,
-		oidc_cache_redis_post_config,
-		oidc_cache_redis_child_init,
-		oidc_cache_redis_get,
-		oidc_cache_redis_set,
-		oidc_cache_redis_destroy
+	"redis",
+	1,
+	oidc_cache_redis_post_config_impl,
+	oidc_cache_redis_child_init,
+	oidc_cache_redis_get,
+	oidc_cache_redis_set,
+	oidc_cache_redis_destroy_impl
 };
