@@ -18,7 +18,7 @@
  */
 
 /***************************************************************************
- * Copyright (C) 2017-2021 ZmartZone Holding BV
+ * Copyright (C) 2017-2022 ZmartZone Holding BV
  * Copyright (C) 2013-2017 Ping Identity Corporation
  * All rights reserved.
  *
@@ -44,16 +44,32 @@
 #ifndef MOD_AUTH_OPENIDC_H_
 #define MOD_AUTH_OPENIDC_H_
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#  undef PACKAGE_NAME
+#  undef PACKAGE_STRING
+#  undef PACKAGE_TARNAME
+#  undef PACKAGE_VERSION
+#  undef PACKAGE_BUGREPORT
+#endif
+
 #include <stdint.h>
 #include <httpd.h>
 #include <http_core.h>
 #include <http_config.h>
+#include <http_log.h>
+#include <http_request.h>
+#include <http_protocol.h>
 #include <mod_auth.h>
 
+#include <apr_base64.h>
+#include <apr_lib.h>
+#include <apr_uuid.h>
+#include <apr_sha1.h>
+
 #include "jose.h"
 #include "cache/cache.h"
 #include "parse.h"
-#include <apr_uuid.h>
 
 #ifdef APLOG_USE_MODULE
 APLOG_USE_MODULE(auth_openidc);
@@ -70,6 +86,7 @@ APLOG_USE_MODULE(auth_openidc);
 
 #define oidc_debug(r, fmt, ...) oidc_log(r, OIDC_DEBUG, fmt, ##__VA_ARGS__)
 #define oidc_warn(r, fmt, ...) oidc_log(r, APLOG_WARNING, fmt, ##__VA_ARGS__)
+#define oidc_info(r, fmt, ...) oidc_log(r, APLOG_INFO, fmt, ##__VA_ARGS__)
 #define oidc_error(r, fmt, ...) oidc_log(r, APLOG_ERR, fmt, ##__VA_ARGS__)
 
 #define oidc_sdebug(s, fmt, ...) oidc_slog(s, OIDC_DEBUG, fmt, ##__VA_ARGS__)
@@ -89,6 +106,7 @@ APLOG_USE_MODULE(auth_openidc);
 #define OIDC_REQUEST_STATE_KEY_CLAIMS  "c"
 #define OIDC_REQUEST_STATE_KEY_DISCOVERY  "d"
 #define OIDC_REQUEST_STATE_KEY_AUTHN  "a"
+#define OIDC_REQUEST_STATE_KEY_SAVE "s"
 
 /* parameter name of the callback URL in the discovery response */
 #define OIDC_DISC_CB_PARAM "oidc_callback"
@@ -202,6 +220,7 @@ APLOG_USE_MODULE(auth_openidc);
 #define OIDC_UNAUTZ_RETURN403    1
 #define OIDC_UNAUTZ_RETURN401    2
 #define OIDC_UNAUTZ_AUTHENTICATE 3
+#define OIDC_UNAUTZ_RETURN302    4
 
 #define OIDC_REQUEST_URI_CACHE_DURATION 30
 
@@ -210,13 +229,13 @@ APLOG_USE_MODULE(auth_openidc);
 
 #define OIDC_COOKIE_EXT_SAME_SITE_LAX    "SameSite=Lax"
 #define OIDC_COOKIE_EXT_SAME_SITE_STRICT "SameSite=Strict"
-#define OIDC_COOKIE_EXT_SAME_SITE_NONE(r) \
-		oidc_util_request_is_secure(r) ? "SameSite=None" : NULL
+#define OIDC_COOKIE_EXT_SAME_SITE_NONE(c, r) \
+		oidc_util_request_is_secure(r,c ) ? "SameSite=None" : NULL
 
 #define OIDC_COOKIE_SAMESITE_STRICT(c, r) \
-		c->cookie_same_site ? OIDC_COOKIE_EXT_SAME_SITE_STRICT : OIDC_COOKIE_EXT_SAME_SITE_NONE(r)
+		c->cookie_same_site ? OIDC_COOKIE_EXT_SAME_SITE_STRICT : OIDC_COOKIE_EXT_SAME_SITE_NONE(c, r)
 #define OIDC_COOKIE_SAMESITE_LAX(c, r) \
-		c->cookie_same_site ? OIDC_COOKIE_EXT_SAME_SITE_LAX : OIDC_COOKIE_EXT_SAME_SITE_NONE(r)
+		c->cookie_same_site ? OIDC_COOKIE_EXT_SAME_SITE_LAX : OIDC_COOKIE_EXT_SAME_SITE_NONE(c, r)
 
 /* https://tools.ietf.org/html/draft-ietf-tokbind-ttrp-01 */
 #define OIDC_TB_CFG_PROVIDED_ENV_VAR     "Sec-Provided-Token-Binding-ID"
@@ -231,6 +250,10 @@ APLOG_USE_MODULE(auth_openidc);
 #define OIDC_STATE_INPUT_HEADERS_USER_AGENT 1
 #define OIDC_STATE_INPUT_HEADERS_X_FORWARDED_FOR 2
 
+#define OIDC_HDR_X_FORWARDED_HOST   1
+#define OIDC_HDR_X_FORWARDED_PORT   2
+#define OIDC_HDR_X_FORWARDED_PROTO  4
+
 typedef apr_byte_t (*oidc_proto_pkce_state)(request_rec *r, char **state);
 typedef apr_byte_t (*oidc_proto_pkce_challenge)(request_rec *r, const char *state, char **code_challenge);
 typedef apr_byte_t (*oidc_proto_pkce_verifier)(request_rec *r, const char *state, char **code_verifier);
@@ -424,10 +447,10 @@ typedef struct oidc_cfg {
 	apr_hash_t *white_listed_claims;
 
 	apr_byte_t state_input_headers;
-
 	apr_hash_t *redirect_urls_allowed;
-
 	char *ca_bundle_path;
+	char *logout_x_frame_options;
+	apr_byte_t x_forwarded_headers;
 } oidc_cfg;
 
 int oidc_check_user_id(request_rec *r);
@@ -456,6 +479,7 @@ apr_byte_t oidc_get_remote_user(request_rec *r, const char *claim_name, const ch
 #define OIDC_REDIRECT_URI_REQUEST_SESSION          "session"
 #define OIDC_REDIRECT_URI_REQUEST_REFRESH          "refresh"
 #define OIDC_REDIRECT_URI_REQUEST_REMOVE_AT_CACHE  "remove_at_cache"
+#define OIDC_REDIRECT_URI_REQUEST_REVOKE_SESSION   "revoke_session"
 #define OIDC_REDIRECT_URI_REQUEST_REQUEST_URI      "request_uri"
 
 // oidc_oauth
@@ -670,6 +694,7 @@ apr_byte_t oidc_proto_handle_authorization_response_idtoken(request_rec *r, oidc
 apr_byte_t oidc_proto_validate_access_token(request_rec *r, oidc_provider_t *provider, oidc_jwt_t *jwt, const char *response_type, const char *access_token);
 apr_byte_t oidc_proto_validate_code(request_rec *r, oidc_provider_t *provider, oidc_jwt_t *jwt, const char *response_type, const char *code);
 apr_byte_t oidc_proto_validate_nonce(request_rec *r, oidc_cfg *cfg, oidc_provider_t *provider, const char *nonce, oidc_jwt_t *jwt);
+apr_byte_t oidc_validate_redirect_url(request_rec *r, oidc_cfg *c, const char *redirect_to_url, apr_byte_t restrict_to_host, char **err_str, char **err_desc);
 
 // oidc_authz.c
 typedef apr_byte_t (*oidc_authz_match_claim_fn_type)(request_rec *, const char * const, const json_t * const);
@@ -724,6 +749,7 @@ apr_array_header_t *oidc_dir_cfg_strip_cookies(request_rec *r);
 int oidc_dir_cfg_unauth_action(request_rec *r);
 apr_byte_t oidc_dir_cfg_unauth_expr_is_set(request_rec *r);
 int oidc_dir_cfg_unautz_action(request_rec *r);
+char *oidc_dir_cfg_unauthz_arg(request_rec *r);
 char *oidc_dir_cfg_path_auth_request_params(request_rec *r);
 char *oidc_dir_cfg_path_scope(request_rec *r);
 oidc_valid_function_t oidc_cfg_get_valid_endpoint_auth_function(oidc_cfg *cfg);
@@ -742,13 +768,13 @@ void oidc_cfg_provider_init(oidc_provider_t *provider);
 int oidc_strnenvcmp(const char *a, const char *b, int len);
 int oidc_base64url_encode(request_rec *r, char **dst, const char *src, int src_len, int remove_padding);
 int oidc_base64url_decode(apr_pool_t *pool, char **dst, const char *src);
-const char *oidc_get_current_url_host(request_rec *r);
-char *oidc_get_current_url(request_rec *r);
+const char *oidc_get_current_url_host(request_rec *r, const apr_byte_t x_forwarded_headers);
+char *oidc_get_current_url(request_rec *r, const apr_byte_t x_forwarded_headers);
 const char *oidc_get_redirect_uri(request_rec *r, oidc_cfg *c);
 const char *oidc_get_redirect_uri_iss(request_rec *r, oidc_cfg *c, oidc_provider_t *provider);
 char *oidc_url_encode(const request_rec *r, const char *str, const char *charsToEncode);
 char *oidc_normalize_header_name(const request_rec *r, const char *str);
-apr_byte_t oidc_util_request_is_secure(request_rec *r);
+apr_byte_t oidc_util_request_is_secure(request_rec *r, oidc_cfg *c);
 void oidc_util_set_cookie(request_rec *r, const char *cookieName, const char *cookieValue, apr_time_t expires, const char *ext);
 char *oidc_util_get_cookie(request_rec *r, const char *cookieName);
 apr_byte_t oidc_util_http_get(request_rec *r, const char *url, const apr_table_t *params, const char *basic_auth, const char *bearer_token, int ssl_validate_server, char **response, int timeout, const char *outgoing_proxy, apr_array_header_t *pass_cookies, const char *ssl_cert, const char *ssl_key, const char *ssl_key_pwd);
@@ -802,29 +828,33 @@ apr_byte_t oidc_enabled(request_rec *r);
 char *oidc_util_http_form_encoded_data(request_rec *r, const apr_table_t *params);
 
 /* HTTP header constants */
-#define OIDC_HTTP_HDR_COOKIE							"Cookie"
-#define OIDC_HTTP_HDR_SET_COOKIE						"Set-Cookie"
-#define OIDC_HTTP_HDR_USER_AGENT						"User-Agent"
-#define OIDC_HTTP_HDR_X_FORWARDED_FOR					"X-Forwarded-For"
-#define OIDC_HTTP_HDR_CONTENT_TYPE						"Content-Type"
-#define OIDC_HTTP_HDR_CONTENT_LENGTH					"Content-Length"
-#define OIDC_HTTP_HDR_X_REQUESTED_WITH					"X-Requested-With"
-#define OIDC_HTTP_HDR_ACCEPT							"Accept"
-#define OIDC_HTTP_HDR_AUTHORIZATION						"Authorization"
-#define OIDC_HTTP_HDR_X_FORWARDED_PROTO					"X-Forwarded-Proto"
-#define OIDC_HTTP_HDR_X_FORWARDED_PORT					"X-Forwarded-Port"
-#define OIDC_HTTP_HDR_X_FORWARDED_HOST					"X-Forwarded-Host"
-#define OIDC_HTTP_HDR_HOST								"Host"
-#define OIDC_HTTP_HDR_LOCATION							"Location"
-#define OIDC_HTTP_HDR_CACHE_CONTROL						"Cache-Control"
-#define OIDC_HTTP_HDR_PRAGMA							"Pragma"
-#define OIDC_HTTP_HDR_P3P								"P3P"
-#define OIDC_HTTP_HDR_EXPIRES							"Expires"
-#define OIDC_HTTP_HDR_X_FRAME_OPTIONS					"X-Frame-Options"
-#define OIDC_HTTP_HDR_WWW_AUTHENTICATE					"WWW-Authenticate"
-#define OIDC_HTTP_HDR_INCLUDE_REFERRED_TOKEN_BINDING_ID	"Include-Referred-Token-Binding-ID"
+#define OIDC_HTTP_HDR_COOKIE                            "Cookie"
+#define OIDC_HTTP_HDR_SET_COOKIE                        "Set-Cookie"
+#define OIDC_HTTP_HDR_USER_AGENT                        "User-Agent"
+#define OIDC_HTTP_HDR_X_FORWARDED_FOR                   "X-Forwarded-For"
+#define OIDC_HTTP_HDR_CONTENT_TYPE                      "Content-Type"
+#define OIDC_HTTP_HDR_CONTENT_LENGTH                    "Content-Length"
+#define OIDC_HTTP_HDR_X_REQUESTED_WITH                  "X-Requested-With"
+#define OIDC_HTTP_HDR_SEC_FETCH_MODE                    "Sec-Fetch-Mode"
+#define OIDC_HTTP_HDR_SEC_FETCH_DEST                    "Sec-Fetch-Dest"
+#define OIDC_HTTP_HDR_ACCEPT                            "Accept"
+#define OIDC_HTTP_HDR_AUTHORIZATION                     "Authorization"
+#define OIDC_HTTP_HDR_X_FORWARDED_PROTO                 "X-Forwarded-Proto"
+#define OIDC_HTTP_HDR_X_FORWARDED_PORT                  "X-Forwarded-Port"
+#define OIDC_HTTP_HDR_X_FORWARDED_HOST                  "X-Forwarded-Host"
+#define OIDC_HTTP_HDR_HOST                              "Host"
+#define OIDC_HTTP_HDR_LOCATION                          "Location"
+#define OIDC_HTTP_HDR_CACHE_CONTROL                     "Cache-Control"
+#define OIDC_HTTP_HDR_PRAGMA                            "Pragma"
+#define OIDC_HTTP_HDR_P3P                               "P3P"
+#define OIDC_HTTP_HDR_EXPIRES                           "Expires"
+#define OIDC_HTTP_HDR_X_FRAME_OPTIONS                   "X-Frame-Options"
+#define OIDC_HTTP_HDR_WWW_AUTHENTICATE                  "WWW-Authenticate"
+#define OIDC_HTTP_HDR_INCLUDE_REFERRED_TOKEN_BINDING_ID "Include-Referred-Token-Binding-ID"
 
-#define OIDC_HTTP_HDR_VAL_XML_HTTP_REQUEST				"XMLHttpRequest"
+#define OIDC_HTTP_HDR_VAL_XML_HTTP_REQUEST "XMLHttpRequest"
+#define OIDC_HTTP_HDR_VAL_NAVIGATE         "navigate"
+#define OIDC_HTTP_HDR_VAL_DOCUMENT         "document"
 
 void oidc_util_hdr_in_set(const request_rec *r, const char *name, const char *value);
 const char *oidc_util_hdr_in_cookie_get(const request_rec *r);
@@ -834,6 +864,8 @@ const char *oidc_util_hdr_in_x_forwarded_for_get(const request_rec *r);
 const char *oidc_util_hdr_in_content_type_get(const request_rec *r);
 const char *oidc_util_hdr_in_content_length_get(const request_rec *r);
 const char *oidc_util_hdr_in_x_requested_with_get(const request_rec *r);
+const char* oidc_util_hdr_in_sec_fetch_mode_get(const request_rec *r);
+const char* oidc_util_hdr_in_sec_fetch_dest_get(const request_rec *r);
 const char *oidc_util_hdr_in_accept_get(const request_rec *r);
 const char *oidc_util_hdr_in_authorization_get(const request_rec *r);
 const char *oidc_util_hdr_in_x_forwarded_proto_get(const request_rec *r);
@@ -858,10 +890,10 @@ apr_byte_t oidc_oauth_metadata_provider_parse(request_rec *r, oidc_cfg *c, json_
 // oidc_session.c
 typedef struct {
 	char uuid[APR_UUID_FORMATTED_LENGTH + 1]; /* unique id */
-    const char *remote_user;                  /* user who owns this particular session */
-    json_t *state;                            /* the state for this session, encoded in a JSON object */
-    apr_time_t expiry;                        /* if > 0, the time of expiry of this session */
-    const char *sid;
+	const char *remote_user;                  /* user who owns this particular session */
+	json_t *state;                            /* the state for this session, encoded in a JSON object */
+	apr_time_t expiry;                        /* if > 0, the time of expiry of this session */
+	const char *sid;
 } oidc_session_t;
 
 apr_byte_t oidc_session_load(request_rec *r, oidc_session_t **z);
@@ -906,7 +938,7 @@ const char * oidc_session_get_session_state(request_rec *r, oidc_session_t *z);
 void oidc_session_set_issuer(request_rec *r, oidc_session_t *z, const char *issuer);
 const char * oidc_session_get_issuer(request_rec *r, oidc_session_t *z);
 void oidc_session_set_client_id(request_rec *r, oidc_session_t *z, const char *client_id);
-
+apr_byte_t oidc_is_auth_capable_request(request_rec *r);
 char *oidc_parse_base64(apr_pool_t *pool, const char *input, char **output, int *output_len);
 
 #endif /* MOD_AUTH_OPENIDC_H_ */
