@@ -18,7 +18,7 @@
  */
 
 /***************************************************************************
- * Copyright (C) 2017-2021 ZmartZone Holding BV
+ * Copyright (C) 2017-2022 ZmartZone Holding BV
  * Copyright (C) 2013-2017 Ping Identity Corporation
  * All rights reserved.
  *
@@ -41,15 +41,7 @@
  * @Author: Hans Zandbelt - hans.zandbelt@zmartzone.eu
  */
 
-#include <apr_lib.h>
-
-#include <httpd.h>
-#include <http_config.h>
-#include <http_log.h>
-#include <http_request.h>
-
 #include "mod_auth_openidc.h"
-#include "parse.h"
 
 apr_byte_t oidc_oauth_metadata_provider_retrieve(request_rec *r, oidc_cfg *cfg,
 		const char *issuer, const char *url, json_t **j_metadata,
@@ -599,6 +591,7 @@ static apr_byte_t oidc_oauth_validate_jwt_access_token(request_rec *r,
 			&err) == FALSE) {
 		oidc_error(r, "could not parse JWT from access_token: %s",
 				oidc_jose_e2s(r->pool, err));
+		oidc_jwk_destroy(jwk);
 		return FALSE;
 	}
 
