@@ -18,7 +18,7 @@
  */
 
 /***************************************************************************
- * Copyright (C) 2017-2021 ZmartZone Holding BV
+ * Copyright (C) 2017-2022 ZmartZone Holding BV
  * Copyright (C) 2013-2017 Ping Identity Corporation
  * All rights reserved.
  *
@@ -43,7 +43,6 @@
  * @Author: Hans Zandbelt - hans.zandbelt@zmartzone.eu
  */
 
-#include <apr_base64.h>
 #include "mod_auth_openidc.h"
 #include "parse.h"
 #include "jose.h"
@@ -1014,6 +1013,7 @@ const char *oidc_parse_unauth_action(apr_pool_t *pool, const char *arg,
 #define OIDC_UNAUTZ_ACTION_AUTH_STR "auth"
 #define OIDC_UNAUTZ_ACTION_401_STR  "401"
 #define OIDC_UNAUTZ_ACTION_403_STR  "403"
+#define OIDC_UNAUTZ_ACTION_302_STR  "302"
 
 /*
  * parse an "unauthorized action" value from the provided string
@@ -1024,6 +1024,7 @@ const char *oidc_parse_unautz_action(apr_pool_t *pool, const char *arg,
 			OIDC_UNAUTZ_ACTION_AUTH_STR,
 			OIDC_UNAUTZ_ACTION_401_STR,
 			OIDC_UNAUTZ_ACTION_403_STR,
+			OIDC_UNAUTZ_ACTION_302_STR,
 			NULL };
 	const char *rv = oidc_valid_string_option(pool, arg, options);
 	if (rv != NULL)
@@ -1035,6 +1036,8 @@ const char *oidc_parse_unautz_action(apr_pool_t *pool, const char *arg,
 		*action = OIDC_UNAUTZ_RETURN401;
 	else if (apr_strnatcmp(arg, OIDC_UNAUTZ_ACTION_403_STR) == 0)
 		*action = OIDC_UNAUTZ_RETURN403;
+	else if (apr_strnatcmp(arg, OIDC_UNAUTZ_ACTION_302_STR) == 0)
+		*action = OIDC_UNAUTZ_RETURN302;
 
 	return NULL;
 }
@@ -1352,3 +1355,25 @@ const char *oidc_parse_set_state_input_headers_as(apr_pool_t *pool, const char *
 
 	return NULL;
 }
+
+const char* oidc_parse_x_forwarded_headers(apr_pool_t *pool, const char *arg,
+		apr_byte_t *x_forwarded_headers) {
+	static char *options[] = {
+			OIDC_HTTP_HDR_X_FORWARDED_HOST,
+			OIDC_HTTP_HDR_X_FORWARDED_PORT,
+			OIDC_HTTP_HDR_X_FORWARDED_PROTO,
+			NULL };
+	const char *rv = oidc_valid_string_option(pool, arg, options);
+	if (rv != NULL)
+		return rv;
+
+	if (apr_strnatcmp(arg, OIDC_HTTP_HDR_X_FORWARDED_HOST) == 0) {
+		*x_forwarded_headers |= OIDC_HDR_X_FORWARDED_HOST;
+	} else if (apr_strnatcmp(arg, OIDC_HTTP_HDR_X_FORWARDED_PORT) == 0) {
+		*x_forwarded_headers |= OIDC_HDR_X_FORWARDED_PORT;
+	} else if (apr_strnatcmp(arg, OIDC_HTTP_HDR_X_FORWARDED_PROTO) == 0) {
+		*x_forwarded_headers |= OIDC_HDR_X_FORWARDED_PROTO;
+	}
+
+	return NULL;
+}
