@@ -37,6 +37,13 @@ AP_DECLARE(long) ap_get_client_block(request_rec * r, char * buffer,
 	return 0;
 }
 
+#if MODULE_MAGIC_NUMBER_MAJOR >= 20100714
+AP_DECLARE(char *) ap_get_exec_line(apr_pool_t *p, const char *cmd,
+		const char * const *argv) {
+	return NULL;
+}
+#endif
+
 AP_DECLARE(char *) ap_getword(apr_pool_t *atrans, const char **line, char stop) {
 	const char *pos = *line;
 	int len;
@@ -144,8 +151,7 @@ AP_DECLARE(int) ap_hook_fixups(request_rec *r) {
 	return 0;
 }
 
-AP_DECLARE(int) ap_hook_insert_filter(request_rec *r) {
-	return 0;
+AP_DECLARE(void) ap_hook_insert_filter(request_rec *r) {
 }
 
 AP_DECLARE(void) ap_hook_post_config(
@@ -298,8 +304,13 @@ AP_DECLARE(void) ap_log_error_(const char *file, int line, int module_index,
 			return NULL;
 		}
 
+		AP_DECLARE(char *) ap_make_dirstr_parent(apr_pool_t *p, const char *s) {
+			return NULL;
+		}
+
+#if MODULE_MAGIC_NUMBER_MAJOR >= 20100714
 		AP_DECLARE(int) ap_expr_exec(request_rec *r, const ap_expr_info_t *expr,
 				const char **err) {
 			return 0;
 		}
-
+#endif
